;-------------------------------------------------------------------
; ImageHide Hidden Data Finder v0.2 (6 December 2003)
; Freeware, Open Source, GPL, Copyleft, whatever you want.
;-------------------------------------------------------------------
;
; It's just a very small utility, quick-and-dirty coded,
; to check if a BMP image contains data "hidden" by ImageHide
; steganography software, recover and decrypt it.
;
; How to use: run my program. Select the BMP file with hidden 
; information. That's all. Several things can happen:
;
;  - if the file is not a 24-bits BMP, you have an error message
;
;  - if the hidden data does not look like a ImageHide
;    hidden data, you have an error message
;
;  - if the hidden data looks like a ImageHide hidden data, 
;    it says so, and this data is saved as a file called
;    "hidden_extracted.txt". At the same time, this data
;    is unencrypted and save in another file called
;    "hidden_extracted_decrypted.txt". I do this because 
;    I cannot know if the hidden text is encrypted or not
;    (actually, I could, by checking the ASCII range of the
;    characters, but I am very lazy tonight), so I save 
;    both versions, and the user will see right away which 
;    one is the plain text.
;    
;
; How the data is recovered on a BMP?
; 
; - go to the last line
; - the first 32 LSB are all 0 (this is used as a 
;   "signature" for ImageHide - of course it can bring
;   false positives, but not false negatives)
; - the 8 next LSB are the message size, low order byte
; - the 8 next LSB are the message size, high order byte
; - the 16 next LSB are all 0 (or maybe the message size,
;   2 next higher order bytes, so all 0 if your message 
;   is smaller than 64 Ko)
; - the next LSB are your message in linear order
  
; Guillermito
; guillermito@pipo.com
; http://www.guillermito2.net
;
; Assemble it with:
;      TASM32 /ml /m3 /z /t ImageHide_Hidden_Text_Finder
;      TLINK32 -Tpe -aa ImageHide_Hidden_Text_Finder,,,import32 

callW macro   x
extrn x:PROC
call x
endm

   .486
   .model flat

.data

openfilename_struct:
lStructSize       dd openfilename_struct_size
hwndOwner         dd 0
hInstance         dd 0
lpstrFilter       dd offset filter
lpstrCustomFilter dd 0
nMaxCustFilter    dd 0
nFilterIndex      dd 0
lpstrFile         dd offset namebuffer
nMaxFile          dd 255
lpstrFileTitle    dd 0
nMaxFileTitle     dd 32
lpstrInitialDir   dd 0
lpstrTitle        dd 0
Flags             dd 1000h+4h+200000h
nFileOffset       dw 0
nFileExtension    dw 0
lpstrDefExt       dd 0
lCustData         dd 0
lpfnHook          dd 0
lpTemplateName    dd 0
openfilename_struct_size equ $-offset openfilename_struct

filter            db "Bmp files *.bmp",0,"*.bmp",0,0
namebuffer	      db 255 dup(0)
result_title      db "ImageHide Hidden Text Finder by Guillermito", 0
no_bmp            db "This file does not have the 24-bits BMP signature",0
no_data           db "This BMP file does not seem to contain hidden data",0
found             db "This BMP file seems to contain data hidden with ImageHide",0ah,0dh,0ah,0dh
                  db "Two files were saved:",0ah,0dh,0ah,0dh
                  db "  - hidden_extracted.txt contains the raw unencrypted data",0ah,0dh,0ah,0dh
                  db "  - hidden_extracted_decrypted.txt contains the decrypted data",0ah,0dh,0ah,0dh
                  db "The hidden message, depending if it was encrypted or not, should be",0ah,0dh
                  db "in one of these two files; the other will contain encrypted crap.",0



file_name          db "hidden_extracted.txt",0
file_name2         db "hidden_extracted_decrypted.txt",0
file_handle2       dd ?
file_size          dd ?
file_handle        dd ?
file_mem_buffer    dd ?
file_nb_bytes_read dd ?
pixels_data_start  dd ?
size_hidden_data   dd ?
counter_end_of_line         dd ?
line_length_with_padding    dd ?
line_length_without_padding dd ?
buffer_for_extraction       dd ?
padding_end_of_line         dd ?
width                       dd ?
heigth                      dd ?
buffer_end                  dd ?

.code

programme:

;-------- choose a file -------------

push offset openfilename_struct
callW GetOpenFileNameA             ;choose file
test eax, eax
jz end

;-------- open it --------------

push 0
push 80h                          ;FILE_ATTRIBUTE_NORMAL
push 3                            ;OPEN_EXISTING
push 0
push 0
push 80000000h+40000000h          ;GENERIC_READ+GENERIC_WRITE
push dword ptr [lpstrFile]
callW CreateFileA                 ;open file
inc eax
jz end
dec eax
mov file_handle, eax

;---------- get its size ---------

push 0
push dword ptr file_handle
callW GetFileSize                 ;get file size in bytes
inc eax
jz close_file
dec eax
mov file_size, eax

;--------- alloc memory for file --------

mov eax, file_size
shl eax, 2
add eax, 1024
push eax                         ;number of bytes to allocate
push 40h                         ;40h=LMEM_ZEROINIT
callW LocalAlloc
test eax, eax
jz close_file
mov file_mem_buffer, eax

;------- read entire file ------------

push 0
push offset file_nb_bytes_read
push file_size
push file_mem_buffer
push file_handle
callW ReadFile
test eax, eax
jz free_memory
mov eax, file_nb_bytes_read
cmp eax, file_size
jnz free_memory

;-------- find BMP signature ---------

mov esi, file_mem_buffer
mov ax, [esi]
cmp ax, "MB"
jne trouble

;------- find number of bits per pixel ------

mov eax, [esi+10]
add eax, file_mem_buffer
mov pixels_data_start, eax
mov eax, [esi+18]
mov width, eax
mov eax, [esi+22]
mov heigth, eax
mov ax, [esi+28]
cmp ax, 24
je this_looks_like_a_bmp

;-------- not 24-bits BMP -----------

trouble:
push 0
push offset result_title
push offset no_bmp
push 0
callW MessageBoxA
jmp free_memory

;------ Prepare the extraction -----------

this_looks_like_a_bmp:

;edi and esi to beginning of new buffer
mov esi, file_mem_buffer
add esi, file_size
mov edi, esi
mov buffer_for_extraction, esi

;calculate line length without padding (width in pixels x 3)
mov eax, width
lea eax, [eax*2+eax]
mov line_length_without_padding, eax
mov counter_end_of_line, eax

;calculate number of padding zero bytes at end of each line
mov eax, line_length_without_padding
and eax, 11b

 jnz there_is_padding
 mov eax, 4
 there_is_padding:

mov ebx, 4
sub ebx, eax
mov padding_end_of_line, ebx

;calculate line length with padding
add ebx, line_length_without_padding
mov line_length_with_padding, ebx

;prepare esi: points to beginning of last line
mov esi, buffer_for_extraction
sub esi, line_length_with_padding

;prepare edi: points to the new buffer
mov edi, buffer_for_extraction

;adjust for ImageHide (for some reason it does not use 2 last bytes)
sub counter_end_of_line, 2
sub line_length_without_padding, 2
add padding_end_of_line, 2

;------------- extract everything --------------

extract_everything:
mov ecx, 0
xor ebx, ebx
xor eax, eax

extract_a_byte:
lodsb				;get byte
and al, 1			;eliminate everything except LSB
rol al, cl			;put the bit where it belongs in the byte
or bl, al			;along with its friends

 ;check if we are at end of line
 dec counter_end_of_line
 jnz not_eol
 ;yes => reset counter
 mov eax, line_length_without_padding
 mov counter_end_of_line, eax
 ;yes => point to the beginning of precedent line
 sub esi, line_length_without_padding
 sub esi, line_length_with_padding
 ;check if we are done
 cmp esi, pixels_data_start
 jae not_eol
 ;yes => save the last unfinished byte and quit
 xchg eax, ebx
 stosb
 jmp break_loop
 not_eol:

inc ecx
cmp ecx, 8
jne extract_a_byte

xchg eax, ebx
stosb
jmp extract_everything
    
break_loop:

;----- calculate how long we have -----

mov buffer_end, edi
sub edi, buffer_for_extraction
mov size_hidden_data, edi

;--- check if hidden data looks like ImageHide structure -----

mov esi, buffer_for_extraction ;start of hidden data buffer

mov eax, [esi]                 ;get the first 32 bits
cmp eax, 0                     ;all 0?
jne no_IH_data_inside          ;no => probably random data

mov eax, [esi+4]               ;yes => get the size of hidden data
mov size_hidden_data, eax      ;and store it

;-------- data found inside -----------

push 0
push offset result_title
push offset found
push 0
callW MessageBoxA
jmp save_file

;-------- no data inside -----------

no_IH_data_inside:
push 0
push offset result_title
push offset no_data
push 0
callW MessageBoxA
jmp free_memory

;------- open a new file on the disk ----------------

save_file:
push 0                  ;handle of file with attributes to copy 
push 80h                ;file attributes (80h=FILE_ATTRIBUTE_NORMAL)
push 2                  ;how to create (2=OPEN_ALWAYS)
push 0                  ;address of security descriptor
push 0                  ;share mode (0=Prevents the file from being shared)
push 40000000h          ;access (read-write) mode (40000000h=GENERIC_WRITE)
push offset file_name   ;address of name of the file
call CreateFileA
mov file_handle2, eax

;------- write buffer in it ----------------

push 0
push offset file_nb_bytes_read  ;address of number of bytes written 
push size_hidden_data           ;number of bytes to write
  mov eax, buffer_for_extraction      ;address of data to write to file
  add eax, 8
  push eax
push file_handle2               ;handle of file to write to
callW WriteFile

;------- close it ----------------

push file_handle2
callW CloseHandle

;------- decrypt the crap -------------

mov esi, buffer_for_extraction
add esi, 8
push esi
pop edi
mov ecx, size_hidden_data
mov ebx, 3d5h
xor eax, eax

decrypt_loop:
lodsb
push eax
push ebx
shr ebx, 8
xor al, bl
pop ebx
stosb
pop eax
add ebx, eax
imul ebx, 3182h
add ebx, 8c33h
loop decrypt_loop

;------- open a new file on the disk ----------------

push 0                  ;handle of file with attributes to copy 
push 80h                ;file attributes (80h=FILE_ATTRIBUTE_NORMAL)
push 2                  ;how to create (2=OPEN_ALWAYS)
push 0                  ;address of security descriptor
push 0                  ;share mode (0=Prevents the file from being shared)
push 40000000h          ;access (read-write) mode (40000000h=GENERIC_WRITE)
push offset file_name2   ;address of name of the file
call CreateFileA
mov file_handle2, eax

;------- write buffer in it ----------------

push 0
push offset file_nb_bytes_read  ;address of number of bytes written 
push size_hidden_data           ;number of bytes to write
  mov eax, buffer_for_extraction      ;address of data to write to file
  add eax, 8
  push eax
push file_handle2               ;handle of file to write to
callW WriteFile

;------- close it ----------------

push file_handle2
callW CloseHandle

;------- close memory and file ----------------

free_memory:
push file_mem_buffer
callW LocalFree

;----------- exit ----------

close_file:
push dword ptr file_handle
callW CloseHandle

end:
push -1
callW ExitProcess

end programme
